from django.core.management.base import BaseCommand, CommandError
from apps.taxonomy.models import TaxonNode, Edge, Tree


class Command(BaseCommand):
    args = '<tree_id tree_id ...>'
    help = 'Populates closure table with given tree nodes. Supports multiple trees.'

    node_id_list = []
    all_tree_edges = []
    initial_query = None

    # Before this command Edge list must be populated
    def handle(self, *args, **options):
        for tree_id in args:
            try:
                tree = Tree.objects.get(pk=int(tree_id))
            except Tree.DoesNotExist:
                raise CommandError('Tree "%s" does not exist')

            root_node = tree.root_node

            root_edge = Edge(ancestor_id=root_node.pk, descendant_id=root_node.pk, length=0)
            root_edge.save()

            self.initial_query = TaxonNode.objects.filter(tree_id=tree_id).filter(valid_name=None)
            for taxon_node in self.initial_query:
                try:
                    longest_edge = Edge.objects.filter(descendant_id=taxon_node.pk).order_by('-length').first().length
                    new_edge = Edge(ancestor_id=root_node.pk, descendant_id=taxon_node.pk, length=longest_edge + 1)
                    new_edge.save()
                except:
                    # print out error message
                    print "No edges for " + str(taxon_node.pk)
